/* _audio.c
 *
 * By W.H.Scholten 2001
 * This file is released to the public domain
 */

#ifdef AUDIO_API__OSS
#include <sys/soundcard.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

//#define PCM_FRAG_SIZE    4096
#define MIN_PCM_SPEED  (22050 - 1000)

//void audio_read_block(void) {
//	for(;;) {
//		audio_bufsize = read(audio_fd, audio_data, PCM_FRAG_SIZE);
//		if (audio_bufsize < 0) {
//			perror("read audio_fd\n");
//			exit(1);
//		}
//		if (audio_bufsize > 0) break;
//	}
//}


int audio_init(char *device, int pcm_speed, int mode) {
	int pcm_channels = 1;
	int pcm_format = AFMT_U8;

	int my_log2_frag = 12; /* 2^12=4096 */
	int my_num_frags = 8;
	int fragvalue = (my_num_frags << 16) | my_log2_frag;

	int audio_fd;

	int ioctl_value;

	if ((audio_fd = open(device, mode, 0)) == -1) {
		perror("Could not open audio device");
		return -1;
	}

	if (ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragvalue) == -1) {
		perror("Unable to set blocksize/number");
		close(audio_fd);
		return -1;
	}

	ioctl_value = pcm_channels - 1; /* 0 = mono, 1 = stereo */
	if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &ioctl_value) == -1) {
		perror("set pcm channels");
		close(audio_fd);
		return -1;
	}

	ioctl_value = pcm_format;
	if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &ioctl_value) == -1) {
		perror("set pcm format");
		close(audio_fd);
		return -1;
	}

	ioctl_value = pcm_speed;
	if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &ioctl_value) == -1) {
		perror("Unable to set pcm speed");
		close(audio_fd);
		return -1;
	}
	if (ioctl_value < MIN_PCM_SPEED) {
		fprintf(stderr, "returned pcm sample frequency too low (%d)", pcm_speed);
		close(audio_fd);
		return -1;
	}

	printf("pcm speed set to %d\n", pcm_speed);

	return audio_fd;
}

#endif /* AUDIO_API__OSS */

