/* _save_file.c, By W.H.Scholten 2001
 *
 * This file is released to the public domain.
 */


int decode_block(bbctape_block *b, unsigned char *d) {
	int  tmp_crc;
	char printable_block_name[ 11 ];

	strncpy(b->name, (char *) (d+1), 11);
	b->name[10] = 0;

	strcpy(printable_block_name, b->name);
	bbcim_printable_bbcfilename(printable_block_name);

	unsigned char *d2 = d + 1 + strlen(b->name) + 1;
	b->load_address = (*d2) | (*(d2+1) << 8) | (*(d2+2) << 16) | (*(d2+3) << 24);
	d2 += 4;
	b->exec_address = (*d2) | (*(d2+1) << 8) | (*(d2+2) << 16) | (*(d2+3) << 24);
	d2 += 4;
	b->number = (*d2) | (*(d2+1) << 8);
	d2 += 2;
	b->length = (*d2) | (*(d2+1) << 8);
	d2 += 2;
	b->flags = (*d2);
	d2 += 1+4; /* skip 4 spare bytes */

	/* 2 byte CRC (MSB): */
	b->header_crc = (*d2 << 8) | (*(d2+1));
	d2 += 2;

	/* Check the header CRC */
	tmp_crc = bbcim_data_crc(d+1, strlen(b->name)+1+19 -2);
	if (b->header_crc != tmp_crc) {
		printf("%s: block %d: header CRC check failed (calculated %4X, file %4X)\n", printable_block_name, b->number, tmp_crc, b->header_crc);
	}
	else {
		//DEB(printf("header CRC ok\n"));
	}

	b->data = d2;

	d2 += b->length;

	/* 2 byte CRC (MSB), only if #bytes>0: */
	if (b->length > 0) {
		b->data_crc = (*d2 << 8) | (*(d2+1));
		d2 += 2;

		/* Check the data CRC */
		tmp_crc = bbcim_data_crc(d + 1+strlen(b->name)+1+19 -2 +2, b->length);
		if (b->data_crc != tmp_crc) {
			printf("%s: block %d: data CRC check failed (calculated %4X, file %4X)\n", printable_block_name, b->number, tmp_crc, b->data_crc);
		}
		else {
			//DEB(printf("data CRC ok\n"));
		}
	}
	return 0;
}


void save_file_blocks(char *blk_basename, int audio) {
	int  i, j;

	u_char *b_data;
	int  b_length;
	struct bbctape_block b;
	char block_filename [ 256 ];

	if ((b_data = (u_char *) malloc(block_size + CASSETTE_MAX_HEADER_SIZE+2)) == NULL) {
		fprintf(stderr, "Out of memory\n");
		exit(1);
	}

	make_wave_table(); /* initialize the sound tables */

	printf("Writing header tone\n");
	make_sync(audio, HEADER_SPACE); /* header tone */


	for (i=0; ; i++) {
		int  source;

		sprintf(block_filename, "%s_%02X", blk_basename, i);
		source = open(block_filename, O_RDONLY);
		if (source < 0) break;

		b_length = lseek(source, 0, SEEK_END);
		lseek(source, 0, SEEK_SET);
		//read(source, b_data, b_length);
		my_read(source, b_data, b_length);
		close(source);

		decode_block(&b, b_data);
		printf("Saving %-10s %02X  (%8lX %8lX %3X %2X)\n", b.name, b.number, b.load_address, b.exec_address, b.length, b.flags);

		for (j=0; j<b_length; j++) {
			save_byte(audio, b_data[j]);
		}
		make_sync(audio, BLOCK_SPACE); /* block sync tone */
	}
	delete_wave_table();
}


void save_file(int source, int audio) {
	int  i;
	u_char *data;
	u_char flags;
	u_short block = 0;
	u_short block_length;
	u_short crc;
	int  bbcfile_length;
	int  bytes_written;
	char printable_blockname[ 11 ];

	if ((data = (u_char *) malloc(block_size)) == NULL) {
		fprintf(stderr, "Out of memory\n");
		exit(1);
	}

	make_wave_table(); /* initialize the sound tables */

	printf("Writing header tone\n");
	make_sync(audio, HEADER_SPACE); /* file header tone */

	bbcfile_length = lseek(source, 0, SEEK_END);
	lseek(source, 0, SEEK_SET);
	bytes_written = 0;

	while ((block_length = my_read(source, data, block_size)) > 0) {
bytes_written += block_length;
		flags = bbcfile_flags;
		if (bytes_written == bbcfile_length) flags |= 0x80;

		/* write the header first: */
		save_byte(audio, (u_char) 0x2a); /* sync byte */

		clear_crc(); /* clear the CRC */

		strcpy(printable_blockname, bbcfile_name);
		bbcim_printable_bbcfilename(printable_blockname);
		printf("Saving ");
		printf("%-10s %02X", printable_blockname, block);
		printf("  (%8lX  %8lX  len %3X  flag %2X)", bbcfile_load_address, bbcfile_exec_address, block_length, flags);
		printf("\n");

		for (i=0; bbcfile_name[i]!='\0'; i++)
			save_byte(audio, bbcfile_name[i]);
		save_byte(audio, (u_char) 0x00);

		save_lsb_long(audio, bbcfile_load_address);
		save_lsb_long(audio, bbcfile_exec_address);
		save_lsb_short(audio, block);
		block++;
		save_lsb_short(audio, block_length);

		save_byte(audio, flags);
		save_lsb_long(audio, 0);

		/* write the header CRC (MSB first): */
		crc = read_crc();
		save_byte(audio, crc >> 8);
		save_byte(audio, crc & 0xff);


		/* write the data */
		clear_crc(); /* clear the CRC */

		if (block_length > 0) {
			for (i=0;i<block_length;i++)
				save_byte(audio, data[i]);

			crc = read_crc();
			save_byte(audio, crc >> 8);
			save_byte(audio, crc & 0xff);
		}

		make_sync(audio, BLOCK_SPACE); /* block sync tone */
	}
	delete_wave_table();
}


void save_tapeimage(char *fn, int audio) {
	FILE *fp;
	unsigned char mybyte;
	int  st;
	int  j = 0; /* visual byte counter */

	make_wave_table(); /* initialize the sound tables */

	printf("Saving tape image:\n");
fprintf(stderr, "save_tapeimage, bit length=%d\n", length);

	fp = fopen(fn, "rb");
	if (fp == NULL) {
		fprintf(stderr, "open %s for reading failed\n", fn);
		exit(1);
	}

	for (;;) {
		st = fread(&mybyte, 1, 1, fp);
		if (st == 1) {
			int  i;
//fprintf(stderr, "save_tapeimage: bytes %d\n", j); j++;
			j++; if (j == 256) { j=0; printf("."); fflush(stdout); }

			for (i=0; i<8; i++) {
				if (mybyte & (1<<i)) { /* write a one */
					if (my_write(audio, one, length) != length) {
						perror(fn);
						exit(1);
					}
				} else {  /* write a zero */
					if (my_write(audio, zero, length) != length) {
						perror(fn);
						exit(1);
					}
				}
			}
		} else break;
	}

	delete_wave_table();
	printf("\nDone\n");
}
