/* bbctape.cc
 *
 * Copyright (C) W.H.Scholten 2001
 */

#define VERSION "0.95"

#include <assert.h>		/* for assert() */
#include <stdlib.h>		/* for exit() */
#include <string.h>		/* for strlen() */
#include <fstream.h>
#include <stdio.h>

#include "bbctape.h"

#include "archive_format/archive_read_attributes.c"

/* read buffer: */
//const int  maxbuflen = 32768;
const int  maxbuflen = 16384;
unsigned char inbuffer[ maxbuflen ];
int  buflen = 0;
int  bufpos = maxbuflen;
int  bufend = 0;

/* Program status: */
bool write_bad_blocks = false;

bool use_audio_in = false;
int  audio_fd;
int  data_fd;

int  sample_freq = BBCTAPE_DEFAULT_SAMPLE_FREQ;

/* For saving: */
char *bbcfile_name;
unsigned long bbcfile_load_address, bbcfile_exec_address;
unsigned char bbcfile_flags;

char *write_file;


int debug = 0; /* verbose output to stderr? */

#include "_io.c" /* read/write wrappers */

#include "_audio.c"

#include "_help.c"

#include "_bbcim_code.c"

#include "_tape_format.c"

#include "_decode.c"
#include "_decode_to_file.c"
#include "_decode_to_image.c"

#include "_save.c"
#include "_save_from_file_image.c"

#define BBCTAPE_MODE__CAT       1
#define BBCTAPE_MODE__EXTRACT   2
#define BBCTAPE_MODE__WRITE     3
int tape_mode = BBCTAPE_MODE__CAT;

#define BBCTAPE_FILE__ARCHIVE   1
#define BBCTAPE_FILE__BLOCKS    2
#define BBCTAPE_FILE__TAPEIMAGE 3
int next_file = BBCTAPE_FILE__ARCHIVE;

#include <signal.h>
void ctrlc_exit(int sig) {
	exit(1);
}

int main(int argc, char **argv)
{
	int  do_save = 0;
	int  do_save_blocks = 0;
	int  do_save_image = 0;
	int  do_write = 0;

	signal(SIGINT, ctrlc_exit);

	if (argc == 1)
		show_help();

	for (int i=1; i<argc; i++) {
		if (strlen(argv[i]) == 0) continue;
		switch (argv[i][0]) {
		case '-':
			if (strlen(argv[i]) == 2) {
				switch (argv[i][1]) {
				case 'D':
					debug = 1;
					break;

				case 'e':
					if (do_save || do_save_blocks || do_save_image || do_write) use();
					do_save = 1;
					break;
				case 'b':
					if (do_save || do_save_blocks || do_save_image || do_write) use();
					do_save_blocks = 1;
					break;
				case 't':
					if (do_save || do_save_blocks || do_save_image || do_write) use();
					do_save_image = 1;
					break;

				case 'w':
					if (do_save || do_save_blocks || do_save_image || do_write) use();
					do_write = 1;
					if (i+1 == argc) use(); /* missing argument */
					tape_mode = BBCTAPE_MODE__WRITE;
					write_file = argv[i+1];
					i++;
					break;
				case 'f':
					write_bad_blocks = true;
					break;

				case 'B':
					next_file = BBCTAPE_FILE__BLOCKS;
					break;

				case 'T':
					next_file = BBCTAPE_FILE__TAPEIMAGE;
					break;

				case '?':
				case 'h':
					show_help();
					break;
				default:
					use();
					break;
				}
			}
			else { /* long options */
				if (i+1 == argc) use(); /* missing argument */

				if (!strcmp(argv[i], "-freq")) {
					sample_freq = atol(argv[i+1]);

					i++;
				}
				else if (!strcmp(argv[i], "-baud")) {
					baud_rate = atol(argv[i+1]);

					i++;
				}
				else if (!strcmp(argv[i], "-bits")) {
					tape_set_bits(argv[i+1]);
					i++;
				}
				else use();
			}
			break;

		default: /* Anything else is a filename */
			if (tape_mode != BBCTAPE_MODE__WRITE) {
				/* READ from file/dsp/stdout (decode/catalogue) ************ */
				if (!strcmp(argv[i], BBCTAPE_AUDIO_DEVICE)) {
//					if (audio_init(argv[i], sample_freq, O_RDONLY) < 0) exit(1);
					audio_fd = audio_init(argv[i], sample_freq, O_RDONLY);
					if (audio_fd < 0) exit(1);
					use_audio_in = true;
				}
				else {
					audio_fd = open(argv[i], O_RDONLY);
					if (audio_fd < 0) {
						perror("Unable to open file");
						exit(1);
					}
				}
				if (do_save_image) {
					decode_to_image("test.tape" /*argv[i+1]*/);
				} else {
					decode_file("", do_save, do_save_blocks);
				}
				close(audio_fd);
				audio_fd = -1;
			}
			else {
				/* WRITE to file/dsp/stdout ******************************** */
				struct bbcfile bfile;

				if (!strcmp(write_file, "-")) {
					/* the file is sent to stdout */
					audio_fd = STDOUT_FILENO;
//printf("stdout\n");
				}
				else if (!strcmp(write_file, BBCTAPE_AUDIO_DEVICE)) {
					audio_fd = audio_init(write_file, sample_freq, O_WRONLY);
					if (audio_fd == -1) {
						exit(-1);
					}
				}
				else {
					audio_fd = open(write_file, O_WRONLY | O_CREAT | O_TRUNC, 0666);
					if (audio_fd == -1) {
						perror(write_file);
						exit(-1);
					}
					lseek(audio_fd, 0, SEEK_END); /* append data */
				}
//printf("%s: audio fd=%d\n", write_file, audio_fd);

				switch (next_file) {
				case BBCTAPE_FILE__ARCHIVE:
					if (bbcim_archive_read_file_attributes(argv[i], &bfile, 1)) {
						fprintf(stderr, "No attributes found for %s\n", argv[i]);
						exit(1);
					}
					bbcfile_load_address = bfile.loadaddr;
					bbcfile_exec_address = bfile.execaddr;
					bbcfile_flags = bfile.attributes;
					bbcfile_name = bfile.name + 2; /* skip the (root or other) directory */
					if ((data_fd = open(argv[i], O_RDONLY, 0)) == -1) { /* read data from file */
						perror(argv[i]);
						exit(-1);
					}
					save_file(data_fd, audio_fd);
					break;

				case BBCTAPE_FILE__BLOCKS:
					save_file_blocks(argv[i], audio_fd);
					break;

				case BBCTAPE_FILE__TAPEIMAGE:
					save_tapeimage(argv[i], audio_fd);
					break;
				default:
					fprintf(stderr, "bad file type %d\n", next_file);
					exit(1);
					break;
 				}

				if (audio_fd != STDOUT_FILENO) close(audio_fd);
				close(data_fd);
				audio_fd = -1;
				next_file = BBCTAPE_FILE__ARCHIVE; /* default */
			}
		}
	}

	return 0;
}
