/*

	MakeUEF by Thomas Harte 2000

	Distributed under the GPL - see 'Copying' for details

*/

char parse_line(char **argv, int argc)
{
	unsigned char help = 0;
	int c, freq;
	char buffer[3];
	char *argument, *end;

	inname = NULL;
	outname = ""; /* dummy */

	buffer[2] = '\0';
	error = NULL;

	i_drv = &card_drv;
	span_type = FREQUENCY_CALIBRATE;
	raw_drv.SetSigned(false);
	raw_drv.SetFrequency(22050);
	get_next_length = get_next_length_sign; /* WHS: peak is now default, as it works much better; but do turn up the volume! */
	bstring = "";
	leadin_type = DUMMYBIT_TONE;

	c = 1;
	while(c < argc)
	{
		memcpy(buffer, argv[c], 2);

		if(buffer[0] == '/' || buffer[0] == '-')
		{
			switch(tolower(buffer[1]))
			{
				default : help = 2; break;
				case 'h' : help = 1; break;
				case 'b' : bstring = "\a"; break;

				case 'f' :
				case 'i' :
				case 't' :
				case 'e' :
				case 's' :
				case 'l' :

					if(strlen(argv[c]) == 2)
					{
						c++;

						if(c < argc)
							argument = argv[c];
						else
						{
							help = 2;
							break;
						}
					}
					else
						argument = argv[c]+2;

					switch(tolower(buffer[1]))
					{
						case 'i' :
							i_drv = &raw_drv;
							inname = argument;
						break;

//						case 'o' :
//							outname = argument;
//						break;

						case 'f' :
							freq = strtol(argument, &end, 0);
							if(*end != '\0')
								help = 2;
							raw_drv.SetFrequency(freq);
						break;

						case 't' :
							switch(tolower(argument[0]))
							{
								case 's' : raw_drv.SetSigned(true); break;
								case 'u' : raw_drv.SetSigned(false); break;
							}
						break;

						case 'e' :
							switch(tolower(argument[0]))
							{
								case 'l' : leadin_type = LONG_TONE; break;
								case 'b' : leadin_type = DUMMYBIT_TONE; break;
							}
						break;

						case 'l' :
							switch(tolower(argument[0]))
							{
								case 'p' : get_next_length = get_next_length_peak; break;
								case 's' : get_next_length = get_next_length_sign; break;
							}
						break;

						case 's' :
							switch(tolower(argument[0]))
							{
								default :
									span_type = WAVE_CALIBRATE;
									span = atol(argument);
								break;

								case 't' :
									span_type = WAVE_CALIBRATE;
								break;

								case 'f' :
									span_type = FREQUENCY_CALIBRATE;
								break;
							}
						break;
					}
				break;
			}
		}
		else
			help = 2;

		c++;
	}

	if(!outname)
	{
		help = 1;
	}
	else
	{
		if(inname)
		{
			argument = inname + strlen(inname);
			while(*argument-- != '.');
			argument+=2;

			if(!strcmp(argument, "wav"))
				i_drv = &wav_drv;
		}
	}

	switch(help)
	{
		case 2 :
			printf("\nINVALID OPTION\n");
		case 1 :
			printf("\n%s [-i <input filename>] [-f <input frequency>] [-h] [-t <u or s>] [-s <t, f or |nn|>] [-l <p or s>] [-b] [-e <b or l>]\n\n", PROGRAM_NAME);
			printf("Converts sound files or soundcard input to UEF tape images.\n");

			#ifdef TARGET_ALLEGRO
			printf("SoundBlaster, AudioDrive, Windows Sound System & Ensoniq Soundscape supported\n");
			#endif
			#ifdef TARGET_OSS
			printf("Open Sound System will be used\n");
			#endif
			#ifdef TARGET_WIN
			printf("Windows sound hardware will be used\n");
			#endif

			printf("\n");
			printf("\t-h brings this message\n");
			printf("\t-f is for raw sound files - takes decimal or hexadecimal (0x) argument\n");
			printf("\t-t is for raw sound files - 'u' for unsigned, 's' for signed.\n");
			printf("\t-s is for time calibration - 't' for by sample, 'f' for by frequency, or supply value\n");
			printf("\t-l sets frequency measure method - s for sign (default), p for peak\n");
			printf("\t-b if this switch is set, a beep occurs on errors\n");
			printf("\t-e determines leadin tone type - l for long, b for with dummy bit\n");
			printf("\n");
			printf("\taccepts '/' in place of '-'\n");
			printf("\tworks with or without spaces between options and parameters\n");
			printf("\twav and raw sound files are supported\n");
			printf("\tif no input name is supplied, the soundcard is used\n");
		break;
	}

	return help;
}
